@@ -43,10 +43,10 @@
 #include <ETL/hermite>
 #include <vector>
 
-#include <synfig/valuenode_bline.h>
-#include <synfig/valuenode_wplist.h>
-#include <synfig/valuenode_dilist.h>
-#include <synfig/valuenode_composite.h>
+#include <synfig/valuenodes/valuenode_bline.h>
+#include <synfig/valuenodes/valuenode_wplist.h>
+#include <synfig/valuenodes/valuenode_dilist.h>
+#include <synfig/valuenodes/valuenode_composite.h>
 
 #endif
 
