@@ -7,6 +7,7 @@
 **	\legal
 **	Copyright (c) 2002-2005 Robert B. Quattlebaum Jr., Adrian Bentley
 **	Copyright (c) 2007 Chris Moore
+**	Copyright (c) 2015 Jérôme Blanchi
 **
 **	This package is free software; you can redistribute it and/or
 **	modify it under the terms of the GNU General Public License as
@@ -34,7 +35,7 @@
 
 #include <ETL/stringf>
 #include "mptr_imagemagick.h"
-#include <stdio.h>
+#include <cstdio>
 #include <sys/types.h>
 #if HAVE_SYS_WAIT_H
  #include <sys/wait.h>
@@ -53,6 +54,7 @@
 #include <functional>
 #include <ETL/stringf>
 #include <synfig/general.h>
+#include <synfig/filesystemnative.h>
 
 #endif
 
@@ -75,16 +77,16 @@ SYNFIG_IMPORTER_SET_NAME(imagemagick_mptr,"imagemagick");
 SYNFIG_IMPORTER_SET_EXT(imagemagick_mptr,"miff");
 SYNFIG_IMPORTER_SET_VERSION(imagemagick_mptr,"0.1");
 SYNFIG_IMPORTER_SET_CVS_ID(imagemagick_mptr,"$Id$");
+SYNFIG_IMPORTER_SET_SUPPORTS_FILE_SYSTEM_WRAPPER(imagemagick_mptr, false);
 
 /* === M E T H O D S ======================================================= */
 
 
-imagemagick_mptr::imagemagick_mptr(const char *f)
-{
-
-	filename=f;
-	file=NULL;
-}
+imagemagick_mptr::imagemagick_mptr(const synfig::FileSystem::Identifier &identifier):
+synfig::Importer(identifier),
+file(NULL),
+cur_frame(0)
+{ }
 
 imagemagick_mptr::~imagemagick_mptr()
 {
@@ -98,13 +100,29 @@ imagemagick_mptr::get_frame(synfig::Surface &surface, const synfig::RendDesc &re
 //#define HAS_LIBPNG 1
 
 #if 1
-	if(filename.empty())
+	if(identifier.filename.empty())
 	{
 		if(cb)cb->error(_("No file to load"));
 		else synfig::error(_("No file to load"));
 		return false;
 	}
 	string temp_file="/tmp/deleteme.png";
+	string temp_container_file="";
+
+	// todo: "container:" and "images" literals
+	if (identifier.filename.substr(0, String("#images/").size())=="#images/")
+	{
+		temp_container_file = "/tmp/synfigtmp.png";
+		// try to copy file to a temp file
+		if (!FileSystem::copy(identifier.file_system, identifier.filename, identifier.file_system, temp_container_file))
+		{
+			if(cb)cb->error(_("Cannot create temporary file of ")+ identifier.filename);
+			else synfig::error(_("Cannot create temporary file of ")+ identifier.filename);
+			return false;
+		}
+	}
+
+	string filename = temp_container_file.size() == 0 ? identifier.filename : temp_container_file;
 
 #if defined(WIN32_PIPE_TO_PROCESSES)
 
@@ -113,7 +131,7 @@ imagemagick_mptr::get_frame(synfig::Surface &surface, const synfig::RendDesc &re
 
 	string command;
 
-	if(filename.find("psd")!=String::npos)
+	if(identifier.filename.find("psd")!=String::npos)
 		command=strprintf("convert \"%s\" -flatten \"png32:%s\"\n",filename.c_str(),temp_file.c_str());
 	else
 		command=strprintf("convert \"%s\" \"png32:%s\"\n",filename.c_str(),temp_file.c_str());
@@ -133,7 +151,7 @@ imagemagick_mptr::get_frame(synfig::Surface &surface, const synfig::RendDesc &re
 
 	if (pid == 0){
 		// Child process
-		if(filename.find("psd")!=String::npos)
+		if(identifier.filename.find("psd")!=String::npos)
 			execlp("convert", "convert", filename.c_str(), "-flatten", output.c_str(), (const char *)NULL);
 		else
 			execlp("convert", "convert", filename.c_str(), output.c_str(), (const char *)NULL);
@@ -150,7 +168,11 @@ imagemagick_mptr::get_frame(synfig::Surface &surface, const synfig::RendDesc &re
 	#error There are no known APIs for creating child processes
 #endif
 
-	Importer::Handle importer(Importer::open(temp_file));
+	//if any delete container tmp file
+	if(temp_container_file.size())
+		identifier.file_system->file_remove(temp_container_file);
+
+	Importer::Handle importer(Importer::open(synfig::FileSystem::Identifier(synfig::FileSystemNative::instance(), temp_file)));
 
 	if(!importer)
 	{
@@ -211,14 +233,14 @@ imagemagick_mptr::get_frame(synfig::Surface &surface, const synfig::RendDesc &re
 
 	string command;
 
-	if(filename.empty())
+	if(identifier.filename.empty())
 	{
 		if(cb)cb->error(_("No file to load"));
 		else synfig::error(_("No file to load"));
 		return false;
 	}
 
-	command=strprintf("convert \"%s\" -flatten ppm:-\n",filename.c_str());
+	command=strprintf("convert \"%s\" -flatten ppm:-\n",identifier.filename.c_str());
 
 	file=popen(command.c_str(),POPEN_BINARY_READ_TYPE);
 
