@@ -92,10 +92,15 @@ Svg_parser::load_svg_canvas(std::string _filepath,String &errors, String &warnin
 	return canvas;
 }
 
-Svg_parser::Svg_parser(){
-	uid=0;
-	kux=60;
-	set_canvas=0;//we must run parser_canvas method
+Svg_parser::Svg_parser():
+	nodeRoot(NULL),
+	uid(0),
+	kux(60),
+	set_canvas(0), //we must run parser_canvas method
+	ox(0),
+	oy(0),
+	locale(LC_NUMERIC, "C")
+{
 	// 0.5 in gamma parameter of color correct layer is 1/0.5 = 2 (thinking) it must be 2.2!!!!
 	gamma.set_gamma(2.2);
 }
@@ -429,7 +434,7 @@ Svg_parser::parser_layer(const xmlpp::Node* node,xmlpp::Element* root,String par
 			layer_style=parent_style;
 		}
 		//build
-		root->set_attribute("type","PasteCanvas");
+		root->set_attribute("type","group");
 		root->set_attribute("active","true");
 		root->set_attribute("version","0.1");
 		if(!label.empty())	root->set_attribute("desc",label);
@@ -1060,7 +1065,7 @@ Svg_parser::build_radialGradient(xmlpp::Element* root,RadialGradient* data,SVGMa
 		if (mtx || data->transform) {
 			xmlpp::Element* layer=root->add_child("layer");
 
-			layer->set_attribute("type","PasteCanvas");
+			layer->set_attribute("type","group");
 			layer->set_attribute("active","true");
 			layer->set_attribute("version","0.1");
 			layer->set_attribute("desc",data->name);
@@ -1128,6 +1133,9 @@ Svg_parser::parser_linearGradient(const xmlpp::Node* node){
 		Glib::ustring link	=nodeElement->get_attribute_value("href");
 		Glib::ustring transform	=nodeElement->get_attribute_value("gradientTransform");
 
+		if(link.empty())
+			link = nodeElement->get_attribute_value("href","xlink");			
+
 		//resolve transformations
 		SVGMatrix* mtx=NULL;
 		if(!transform.empty())
@@ -1178,6 +1186,9 @@ Svg_parser::parser_radialGradient(const xmlpp::Node* node){
 		Glib::ustring link	=nodeElement->get_attribute_value("href");//basic
 		Glib::ustring transform	=nodeElement->get_attribute_value("gradientTransform");
 
+		if(link.empty())
+			link = nodeElement->get_attribute_value("href","xlink");
+
 		if (cx!=fx || cy!=fy)
 			std::cout<<"SVG Parser: ignoring focus attributes for radial gradient";
 
@@ -1440,7 +1451,7 @@ Svg_parser::build_vector (xmlpp::Element* root,String name,float x,float y,Strin
 
 xmlpp::Element*
 Svg_parser::nodeStartBasicLayer(xmlpp::Element* root, String name){
-	root->set_attribute("type","PasteCanvas");
+	root->set_attribute("type","group");
 	root->set_attribute("active","true");
 	root->set_attribute("version","0.1");
 	root->set_attribute("desc",name);
