@@ -38,29 +38,34 @@
 #include "savecanvas.h"
 #include "general.h"
 #include "valuenode.h"
-#include "valuenode_animated.h"
-#include "valuenode_const.h"
-#include "valuenode_linear.h"
-#include "valuenode_staticlist.h"
-#include "valuenode_dynamiclist.h"
-#include "valuenode_reference.h"
-#include "valuenode_subtract.h"
-#include "valuenode_bline.h"
-#include "valuenode_bone.h"
-#include "valuenode_wplist.h"
-#include "valuenode_dilist.h"
+#include "valuenodes/valuenode_animated.h"
+#include "valuenodes/valuenode_const.h"
+#include "valuenodes/valuenode_linear.h"
+#include "valuenodes/valuenode_staticlist.h"
+#include "valuenodes/valuenode_dynamiclist.h"
+#include "valuenodes/valuenode_reference.h"
+#include "valuenodes/valuenode_subtract.h"
+#include "valuenodes/valuenode_bline.h"
+#include "valuenodes/valuenode_bone.h"
+#include "valuenodes/valuenode_wplist.h"
+#include "valuenodes/valuenode_dilist.h"
 #include "dashitem.h"
 #include "time.h"
 #include "keyframe.h"
 #include "layer.h"
 #include "string.h"
 #include "paramdesc.h"
+#include "weightedvalue.h"
+#include "pair.h"
+
+#include "zstreambuf.h"
+#include "importer.h"
+#include "cairoimporter.h"
 
 #include <libxml++/libxml++.h>
 #include <ETL/stringf>
 #include "gradient.h"
 #include <errno.h>
-#include <sys/stat.h>
 
 extern "C" {
 #include <libxml/tree.h>
@@ -85,6 +90,8 @@ using namespace synfig;
 
 ReleaseVersion save_canvas_version = ReleaseVersion(RELEASE_VERSION_END-1);
 int valuenode_too_new_count;
+save_canvas_external_file_callback_t save_canvas_external_file_callback = NULL;
+void *save_canvas_external_file_user_data = NULL;
 
 /* === P R O C E D U R E S ================================================= */
 
@@ -214,12 +221,12 @@ xmlpp::Element* encode_segment(xmlpp::Element* root,Segment seg)
 
 xmlpp::Element* encode_bline_point(xmlpp::Element* root,BLinePoint bline_point)
 {
-	root->set_name(ValueBase::type_name(ValueBase::TYPE_BLINEPOINT));
+	root->set_name(type_bline_point.description.name);
 
 	encode_vector(root->add_child("vertex")->add_child("vector"),bline_point.get_vertex());
 	encode_vector(root->add_child("t1")->add_child("vector"),bline_point.get_tangent1());
 
-	if(bline_point.get_split_tangent_flag())
+	if(bline_point.get_split_tangent_both())
 		encode_vector(root->add_child("t2")->add_child("vector"),bline_point.get_tangent2());
 
 	encode_real(root->add_child("width")->add_child("real"),bline_point.get_width());
@@ -229,7 +236,7 @@ xmlpp::Element* encode_bline_point(xmlpp::Element* root,BLinePoint bline_point)
 
 xmlpp::Element* encode_width_point(xmlpp::Element* root,WidthPoint width_point)
 {
-	root->set_name(ValueBase::type_name(ValueBase::TYPE_WIDTHPOINT));
+	root->set_name(type_width_point.description.name);
 	encode_real(root->add_child("position")->add_child("real"),width_point.get_position());
 	encode_real(root->add_child("width")->add_child("real"),width_point.get_width());
 	encode_integer(root->add_child("side_before")->add_child("integer"),width_point.get_side_type_before());
@@ -239,7 +246,7 @@ xmlpp::Element* encode_width_point(xmlpp::Element* root,WidthPoint width_point)
 
 xmlpp::Element* encode_dash_item(xmlpp::Element* root, DashItem dash_item)
 {
-	root->set_name(ValueBase::type_name(ValueBase::TYPE_DASHITEM));
+	root->set_name(type_dash_item.description.name);
 	encode_real(root->add_child("offset")->add_child("real"),dash_item.get_offset());
 	encode_real(root->add_child("length")->add_child("real"),dash_item.get_length());
 	encode_integer(root->add_child("side_before")->add_child("integer"),dash_item.get_side_type_before());
@@ -263,86 +270,141 @@ xmlpp::Element* encode_gradient(xmlpp::Element* root,Gradient x)
 
 xmlpp::Element* encode_value(xmlpp::Element* root,const ValueBase &data,Canvas::ConstHandle canvas=0);
 
-xmlpp::Element* encode_list(xmlpp::Element* root,std::list<ValueBase> list, Canvas::ConstHandle canvas=0)
+xmlpp::Element* encode_list(xmlpp::Element* root,std::vector<ValueBase> list, Canvas::ConstHandle canvas=0)
 {
 	root->set_name("list");
 
 	while(!list.empty())
 	{
 		encode_value(root->add_child("value"),list.front(),canvas);
-		list.pop_front();
+		list.erase(list.begin());
 	}
 
 	return root;
 }
 
+xmlpp::Element* encode_transformation(xmlpp::Element* root,const Transformation &transformation)
+{
+	root->set_name("transformation");
+	encode_vector(root->add_child("offset")->add_child("vector"),transformation.offset);
+	encode_angle(root->add_child("angle")->add_child("angle"),transformation.angle);
+	encode_angle(root->add_child("skew_angle")->add_child("angle"),transformation.skew_angle);
+	encode_vector(root->add_child("scale")->add_child("vector"),transformation.scale);
+	return root;
+}
+
+xmlpp::Element* encode_weighted_value(xmlpp::Element* root,types_namespace::TypeWeightedValueBase &type, const ValueBase &data,Canvas::ConstHandle canvas)
+{
+	root->set_name(type.description.name);
+	encode_real(root->add_child("weight")->add_child("real"), type.extract_weight(data));
+	encode_value(root->add_child("value")->add_child("value"), type.extract_value(data), canvas);
+	return root;
+}
+
+xmlpp::Element* encode_pair(xmlpp::Element* root,types_namespace::TypePairBase &type, const ValueBase &data,Canvas::ConstHandle canvas)
+{
+	root->set_name(type.description.name);
+	encode_value(root->add_child("first")->add_child("value"), type.extract_first(data), canvas);
+	encode_value(root->add_child("second")->add_child("value"), type.extract_second(data), canvas);
+	return root;
+}
+
 xmlpp::Element* encode_value(xmlpp::Element* root,const ValueBase &data,Canvas::ConstHandle canvas)
 {
-	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_value (type %s)\n", __FILE__, __LINE__, ValueBase::type_name(data.get_type()).c_str());
-	switch(data.get_type())
+	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_value (type %s)\n", __FILE__, __LINE__, data.get_type().description.name.c_str());
+	Type &type(data.get_type());
+	if (type == type_real)
 	{
-	case ValueBase::TYPE_REAL:
 		encode_real(root,data.get(Real()));
 		encode_static(root, data.get_static());
 		encode_interpolation(root, data.get_interpolation(), "interpolation");
 		return root;
-	case ValueBase::TYPE_TIME:
+	}
+	if (type == type_time)
+	{
 		encode_time(root,data.get(Time()));
 		encode_static(root, data.get_static());
 		encode_interpolation(root, data.get_interpolation(), "interpolation");
 		return root;
-	case ValueBase::TYPE_INTEGER:
+	}
+	if (type == type_integer)
+	{
 		encode_integer(root,data.get(int()));
 		encode_static(root, data.get_static());
 		encode_interpolation(root, data.get_interpolation(), "interpolation");
 		return root;
-	case ValueBase::TYPE_COLOR:
+	}
+	if (type == type_color)
+	{
 		encode_color(root,data.get(Color()));
 		encode_static(root, data.get_static());
 		encode_interpolation(root, data.get_interpolation(), "interpolation");
 		return root;
-	case ValueBase::TYPE_VECTOR:
+	}
+	if (type == type_vector)
+	{
 		encode_vector(root,data.get(Vector()));
 		encode_static(root, data.get_static());
 		encode_interpolation(root, data.get_interpolation(), "interpolation");
 		return root;
-	case ValueBase::TYPE_ANGLE:
+	}
+	if (type == type_angle)
+	{
 		encode_angle(root,data.get(Angle()));
 		encode_static(root, data.get_static());
 		encode_interpolation(root, data.get_interpolation(), "interpolation");
 		return root;
-	case ValueBase::TYPE_BOOL:
+	}
+	if (type == type_bool)
+	{
 		encode_bool(root,data.get(bool()));
 		encode_static(root, data.get_static());
 		encode_interpolation(root, data.get_interpolation(), "interpolation");
 		return root;
-	case ValueBase::TYPE_STRING:
+	}
+	if (type == type_string)
+	{
 		encode_string(root,data.get(String()));
 		encode_static(root, data.get_static());
 		encode_interpolation(root, data.get_interpolation(), "interpolation");
 		return root;
-	case ValueBase::TYPE_SEGMENT:
+	}
+	if (type == type_segment)
+	{
 		encode_segment(root,data.get(Segment()));
 		encode_static(root, data.get_static());
 		encode_interpolation(root, data.get_interpolation(), "interpolation");
 		return root;
-	case ValueBase::TYPE_BLINEPOINT:
+	}
+	if (type == type_bline_point)
 		return encode_bline_point(root,data.get(BLinePoint()));
-	case ValueBase::TYPE_WIDTHPOINT:
+	if (type == type_width_point)
 		return encode_width_point(root,data.get(WidthPoint()));
-	case ValueBase::TYPE_DASHITEM:
+	if (type == type_dash_item)
 		return encode_dash_item(root,data.get(DashItem()));
-	case ValueBase::TYPE_GRADIENT:
+	if (type == type_gradient)
+	{
 		encode_gradient(root,data.get(Gradient()));
 		encode_static(root, data.get_static());
 		encode_interpolation(root, data.get_interpolation(), "interpolation");
 		return root;
-	case ValueBase::TYPE_LIST:
-		return encode_list(root,data,canvas);
-	case ValueBase::TYPE_CANVAS:
-		return encode_canvas(root,data.get(Canvas::Handle()).get());
+	}
+	if (type == type_transformation)
+	{
+		encode_transformation(root,data.get(Transformation()));
 		encode_static(root, data.get_static());
-	case ValueBase::TYPE_VALUENODE_BONE:
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
+	}
+	if (type == type_list)
+		return encode_list(root,data.get_list(),canvas);
+	if (type == type_canvas)
+	{
+		return encode_canvas(root,data.get(Canvas::Handle()).get());
+		//encode_static(root, data.get_static());
+	}
+	if (type == type_bone_valuenode)
+	{
 		if (!canvas)
 		{
 			printf("%s:%d ------------------------------------------------------------------------\n", __FILE__, __LINE__);
@@ -352,15 +414,31 @@ xmlpp::Element* encode_value(xmlpp::Element* root,const ValueBase &data,Canvas::
 		root = encode_value_node_bone_id(root,data.get(ValueNode_Bone::Handle()).get(),canvas);
 		root->set_name("bone_valuenode");
 		return root;
-	case ValueBase::TYPE_NIL:
+	}
+	if (dynamic_cast<types_namespace::TypeWeightedValueBase*>(&type) != NULL)
+	{
+		encode_weighted_value(root, *dynamic_cast<types_namespace::TypeWeightedValueBase*>(&type), data, canvas);
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
+	}
+	if (dynamic_cast<types_namespace::TypePairBase*>(&type) != NULL)
+	{
+		encode_pair(root, *dynamic_cast<types_namespace::TypePairBase*>(&type), data, canvas);
+		encode_static(root, data.get_static());
+		encode_interpolation(root, data.get_interpolation(), "interpolation");
+		return root;
+	}
+	if (type == type_nil)
+	{
 		synfig::error("Encountered NIL ValueBase");
 		root->set_name("nil");
 		return root;
-	default:
-		synfig::error(strprintf(_("Unknown value(%s), cannot create XML representation!"),ValueBase::type_local_name(data.get_type()).c_str()));
-		root->set_name("nil");
-		return root;
 	}
+
+	synfig::error(strprintf(_("Unknown value(%s), cannot create XML representation!"), data.get_type().description.local_name.c_str()));
+	root->set_name("nil");
+	return root;
 }
 
 xmlpp::Element* encode_animated(xmlpp::Element* root,ValueNode_Animated::ConstHandle value_node,Canvas::ConstHandle canvas=0)
@@ -368,7 +446,7 @@ xmlpp::Element* encode_animated(xmlpp::Element* root,ValueNode_Animated::ConstHa
 	assert(value_node);
 	root->set_name("animated");
 
-	root->set_attribute("type",ValueBase::type_name(value_node->get_type()));
+	root->set_attribute("type",value_node->get_type().description.name);
 
 	const ValueNode_Animated::WaypointList &waypoint_list=value_node->waypoint_list();
 	ValueNode_Animated::WaypointList::const_iterator iter;
@@ -387,7 +465,7 @@ xmlpp::Element* encode_animated(xmlpp::Element* root,ValueNode_Animated::ConstHa
 			if(ValueNode_Const::ConstHandle::cast_dynamic(value_node))
 			{
 				const ValueBase data = ValueNode_Const::ConstHandle::cast_dynamic(value_node)->get_value();
-				if (data.get_type() == ValueBase::TYPE_CANVAS)
+				if (data.get_type() == type_canvas)
 					waypoint_node->set_attribute("use",data.get(Canvas::Handle()).get()->get_relative_id(canvas));
 				else
 					encode_value_node(waypoint_node->add_child("value_node"),iter->get_value_node(),canvas);
@@ -433,7 +511,7 @@ xmlpp::Element* encode_subtract(xmlpp::Element* root,ValueNode_Subtract::ConstHa
 	assert(lhs);
 	assert(rhs);
 
-	root->set_attribute("type",ValueBase::type_name(value_node->get_type()));
+	root->set_attribute("type",value_node->get_type().description.name);
 
 	if(lhs==rhs)
 		warning("LHS is equal to RHS, this <subtract> will always be zero!");
@@ -466,7 +544,7 @@ xmlpp::Element* encode_static_list(xmlpp::Element* root,ValueNode_StaticList::Co
 
 	root->set_name(value_node->get_name());
 
-	root->set_attribute("type",ValueBase::type_name(value_node->get_contained_type()));
+	root->set_attribute("type",value_node->get_contained_type().description.name);
 
 	vector<ValueNode::RHandle>::const_iterator iter;
 
@@ -494,7 +572,7 @@ xmlpp::Element* encode_dynamic_list(xmlpp::Element* root,ValueNode_DynamicList::
 
 	root->set_name(value_node->get_name());
 
-	root->set_attribute("type",ValueBase::type_name(value_node->get_contained_type()));
+	root->set_attribute("type",value_node->get_contained_type().description.name);
 
 	vector<ValueNode_DynamicList::ListEntry>::const_iterator iter;
 
@@ -606,7 +684,7 @@ xmlpp::Element* encode_linkable_value_node(xmlpp::Element* root,LinkableValueNod
 
 	root->set_name(name);
 
-	root->set_attribute("type",ValueBase::type_name(value_node->get_type()));
+	root->set_attribute("type",value_node->get_type().description.name);
 
 	int i;
 	synfig::ParamVocab child_vocab(value_node->get_children_vocab());
@@ -724,7 +802,7 @@ xmlpp::Element* encode_value_node_bone(xmlpp::Element* root,ValueNode::ConstHand
 xmlpp::Element* encode_value_node_bone_id(xmlpp::Element* root,ValueNode::ConstHandle value_node,Canvas::ConstHandle canvas)
 {
 	root->set_name("bone");
-	root->set_attribute("type",ValueBase::type_name(ValueBase::TYPE_BONE));
+	root->set_attribute("type",type_bone_object.description.name);
 	if (getenv("SYNFIG_DEBUG_SAVE_CANVAS")) printf("%s:%d encode_value_node_bone_id %s %s\n", __FILE__, __LINE__, value_node->get_string().c_str(), value_node->get_guid().get_string().c_str());
 	if(!value_node->get_id().empty())
 		root->set_attribute("id",value_node->get_id());
@@ -794,7 +872,7 @@ xmlpp::Element* encode_layer(xmlpp::Element* root,Layer::ConstHandle layer)
 				continue;
 			}
 
-			if(value.get_type()==ValueBase::TYPE_CANVAS)
+			if(value.get_type()==type_canvas)
 			{
 				// the ->is_inline() below was crashing if the canvas
 				// contained a PasteCanvas with the default <No Image
@@ -819,6 +897,28 @@ xmlpp::Element* encode_layer(xmlpp::Element* root,Layer::ConstHandle layer)
 			xmlpp::Element *node=root->add_child("param");
 			node->set_attribute("name",iter->get_name());
 
+			// remember filename param if need
+			if (save_canvas_external_file_callback != NULL
+			 && iter->get_name() == "filename"
+			 && value.get_type() == type_string)
+			{
+				std::string filename( value.get(String()) );
+				std::string ext = filename_extension(filename);
+				if (!ext.empty()) ext = ext.substr(1); // skip initial '.'
+				bool registered_in_importer = Importer::book().count(ext) > 0;
+				bool supports_by_importer = registered_in_importer
+						                 && Importer::book()[ext].supports_file_system_wrapper;
+				bool registered_in_cairoimporter = CairoImporter::book().count(ext) > 0;
+				bool supports_by_cairoimporter = registered_in_cairoimporter
+						                      && CairoImporter::book()[ext].supports_file_system_wrapper;
+				bool supports = (supports_by_importer && supports_by_cairoimporter)
+						     || (supports_by_importer && !registered_in_cairoimporter)
+						     || (!registered_in_importer && supports_by_cairoimporter);
+				if (supports)
+					if (save_canvas_external_file_callback(save_canvas_external_file_user_data, layer, iter->get_name(), filename))
+						value.set(filename);
+			}
+
 			encode_value(node->add_child("value"),value,layer->get_canvas().constant());
 		}
 	}
@@ -973,16 +1073,11 @@ xmlpp::Element* encode_canvas_toplevel(xmlpp::Element* root,Canvas::ConstHandle
 }
 
 bool
-synfig::save_canvas(const String &filename, Canvas::ConstHandle canvas)
+synfig::save_canvas(const FileSystem::Identifier &identifier, Canvas::ConstHandle canvas, bool safe)
 {
     ChangeLocale change_locale(LC_NUMERIC, "C");
 
-	synfig::String tmp_filename(filename+".TMP");
-
-	if (filename_extension(filename) == ".sifz")
-		xmlSetCompressMode(9);
-	else
-		xmlSetCompressMode(0);
+    synfig::String tmp_filename(safe ? identifier.filename+".TMP" : identifier.filename);
 
 	try
 	{
@@ -991,34 +1086,29 @@ synfig::save_canvas(const String &filename, Canvas::ConstHandle canvas)
 
 		encode_canvas_toplevel(document.create_root_node("canvas"),canvas);
 
-		document.write_to_file_formatted(tmp_filename);
+		FileSystem::WriteStreamHandle stream = identifier.file_system->get_write_stream(tmp_filename);
+		if (!stream)
+		{
+			synfig::error("synfig::save_canvas(): Unable to open file for write");
+			return false;
+		}
 
-#ifdef _WIN32
-		// On Win32 platforms, rename() has bad behavior. Work around it.
-		
-		// Make random filename and ensure there's no file with such name exist
-		struct stat buf;
-		String old_file;
-		do {
-			synfig::GUID guid;
-			old_file = filename+"."+guid.get_string().substr(0,8);
-		} while (stat(old_file.c_str(), &buf) != -1);
-		
-		rename(filename.c_str(),old_file.c_str());
-		if(rename(tmp_filename.c_str(),filename.c_str())!=0)
+		if (filename_extension(identifier.filename) == ".sifz")
+			stream = FileSystem::WriteStreamHandle(new ZWriteStream(stream));
+
+		document.write_to_stream_formatted(*stream, "UTF-8");
+
+		// close stream
+		stream.reset();
+
+		if (safe)
 		{
-			rename(old_file.c_str(),tmp_filename.c_str());
-			synfig::error("synfig::save_canvas(): Unable to rename file to correct filename, errno=%d",errno);
-			return false;
+			if(!identifier.file_system->file_rename(tmp_filename, identifier.filename))
+			{
+				synfig::error("synfig::save_canvas(): Unable to rename file to correct filename");
+				return false;
+			}
 		}
-		remove(old_file.c_str());
-#else
-		if(rename(tmp_filename.c_str(),filename.c_str())!=0)
-		{
-			synfig::error("synfig::save_canvas(): Unable to rename file to correct filename, errno=%d",errno);
-			return false;
-		}
-#endif
 	}
 	catch(...) { synfig::error("synfig::save_canvas(): Caught unknown exception"); return false; }
 
@@ -1038,6 +1128,13 @@ synfig::canvas_to_string(Canvas::ConstHandle canvas)
 	return document.write_to_string_formatted();
 }
 
+void
+synfig::set_save_canvas_external_file_callback(save_canvas_external_file_callback_t callback, void *user_data)
+{
+	save_canvas_external_file_callback = callback;
+	save_canvas_external_file_user_data = user_data;
+}
+
 void
 synfig::set_file_version(ReleaseVersion version)
 {
