@@ -33,20 +33,31 @@ namespace synfig {
 
 struct TargetParam
 {
-	//! Default constructor
+	//Spritesheet render direction
+	enum Direction {
+		HR = 0, //Horisontal 
+		VR = 1  //Vertical
+	};
+	
+	//! Constructor
 	/*! Not valid default values, if they are not modified before
 	 *  passing them to the target module, it would override them with
 	 *  its own valid default settings.
 	 */
-	TargetParam (): video_codec("none"), bitrate(-1), sequence_separator(".") { }
-
-	TargetParam (const std::string& Video_codec, int Bitrate):
-		video_codec(Video_codec), bitrate(Bitrate)
+	TargetParam (const std::string& Video_codec = "none", int Bitrate = -1):
+		video_codec(Video_codec), bitrate(Bitrate), sequence_separator("."), offset_x(0), offset_y(0),rows(0),columns(0),append(true),dir(HR)
 	{ }
 
 	std::string video_codec;
 	int bitrate;
 	std::string sequence_separator;
+	//TODO: It is a spike. Need to separate this class.
+	int offset_x;
+	int offset_y;
+	int rows;
+	int columns;
+	bool append;
+	Direction dir;
 };
 
 }; // END of namespace synfig
