@@ -36,63 +36,68 @@
 #include "canvas.h"
 #include "releases.h"
 
-#include "valuenode_const.h"
-#include "valuenode_linear.h"
-#include "valuenode_composite.h"
-#include "valuenode_reference.h"
-#include "valuenode_boneinfluence.h"
-#include "valuenode_boneweightpair.h"
-#include "valuenode_bone.h"
-#include "valuenode_greyed.h"
-#include "valuenode_scale.h"
-#include "valuenode_blinecalctangent.h"
-#include "valuenode_blinecalcvertex.h"
-#include "valuenode_blinecalcwidth.h"
-#include "valuenode_blinereversetangent.h"
-#include "valuenode_segcalctangent.h"
-#include "valuenode_segcalcvertex.h"
-#include "valuenode_repeat_gradient.h"
-#include "valuenode_stripes.h"
-#include "valuenode_range.h"
-#include "valuenode_add.h"
-#include "valuenode_subtract.h"
-#include "valuenode_timedswap.h"
-#include "valuenode_twotone.h"
-#include "valuenode_bline.h"
-#include "valuenode_staticlist.h"
-#include "valuenode_wplist.h"
-#include "valuenode_dilist.h"
-#include "valuenode_dynamiclist.h"
-#include "valuenode_radialcomposite.h"
-#include "valuenode_gradientrotate.h"
-#include "valuenode_sine.h"
-#include "valuenode_cos.h"
-#include "valuenode_atan2.h"
-#include "valuenode_exp.h"
-#include "valuenode_switch.h"
-#include "valuenode_timeloop.h"
-#include "valuenode_reciprocal.h"
-#include "valuenode_duplicate.h"
-#include "valuenode_integer.h"
-#include "valuenode_step.h"
-#include "valuenode_vectorangle.h"
-#include "valuenode_vectorlength.h"
-#include "valuenode_vectorx.h"
-#include "valuenode_vectory.h"
-#include "valuenode_gradientcolor.h"
-#include "valuenode_dotproduct.h"
-#include "valuenode_timestring.h"
-#include "valuenode_realstring.h"
-#include "valuenode_join.h"
-#include "valuenode_anglestring.h"
-#include "valuenode_intstring.h"
-#include "valuenode_log.h"
-#include "valuenode_pow.h"
-#include "valuenode_compare.h"
-#include "valuenode_not.h"
-#include "valuenode_and.h"
-#include "valuenode_or.h"
-#include "valuenode_real.h"
+#include "valuenodes/valuenode_const.h"
+#include "valuenodes/valuenode_linear.h"
+#include "valuenodes/valuenode_composite.h"
+#include "valuenodes/valuenode_reference.h"
+#include "valuenodes/valuenode_boneinfluence.h"
+#include "valuenodes/valuenode_boneweightpair.h"
+#include "valuenodes/valuenode_bone.h"
+#include "valuenodes/valuenode_bonelink.h"
+#include "valuenodes/valuenode_greyed.h"
+#include "valuenodes/valuenode_scale.h"
+#include "valuenodes/valuenode_blinecalctangent.h"
+#include "valuenodes/valuenode_blinecalcvertex.h"
+#include "valuenodes/valuenode_blinecalcwidth.h"
+#include "valuenodes/valuenode_blinereversetangent.h"
+#include "valuenodes/valuenode_segcalctangent.h"
+#include "valuenodes/valuenode_segcalcvertex.h"
+#include "valuenodes/valuenode_repeat_gradient.h"
+#include "valuenodes/valuenode_stripes.h"
+#include "valuenodes/valuenode_range.h"
+#include "valuenodes/valuenode_add.h"
+#include "valuenodes/valuenode_subtract.h"
+#include "valuenodes/valuenode_timedswap.h"
+#include "valuenodes/valuenode_twotone.h"
+#include "valuenodes/valuenode_bline.h"
+#include "valuenodes/valuenode_staticlist.h"
+#include "valuenodes/valuenode_wplist.h"
+#include "valuenodes/valuenode_dilist.h"
+#include "valuenodes/valuenode_dynamiclist.h"
+#include "valuenodes/valuenode_radialcomposite.h"
+#include "valuenodes/valuenode_gradientrotate.h"
+#include "valuenodes/valuenode_sine.h"
+#include "valuenodes/valuenode_cos.h"
+#include "valuenodes/valuenode_atan2.h"
+#include "valuenodes/valuenode_exp.h"
+#include "valuenodes/valuenode_switch.h"
+#include "valuenodes/valuenode_timeloop.h"
+#include "valuenodes/valuenode_reciprocal.h"
+#include "valuenodes/valuenode_duplicate.h"
+#include "valuenodes/valuenode_integer.h"
+#include "valuenodes/valuenode_step.h"
+#include "valuenodes/valuenode_vectorangle.h"
+#include "valuenodes/valuenode_vectorlength.h"
+#include "valuenodes/valuenode_vectorx.h"
+#include "valuenodes/valuenode_vectory.h"
+#include "valuenodes/valuenode_gradientcolor.h"
+#include "valuenodes/valuenode_dotproduct.h"
+#include "valuenodes/valuenode_timestring.h"
+#include "valuenodes/valuenode_realstring.h"
+#include "valuenodes/valuenode_join.h"
+#include "valuenodes/valuenode_anglestring.h"
+#include "valuenodes/valuenode_intstring.h"
+#include "valuenodes/valuenode_log.h"
+#include "valuenodes/valuenode_pow.h"
+#include "valuenodes/valuenode_compare.h"
+#include "valuenodes/valuenode_not.h"
+#include "valuenodes/valuenode_and.h"
+#include "valuenodes/valuenode_or.h"
+#include "valuenodes/valuenode_real.h"
+#include "valuenodes/valuenode_average.h"
+#include "valuenodes/valuenode_dynamic.h"
+#include "valuenodes/valuenode_derivative.h"
+#include "valuenodes/valuenode_weightedaverage.h"
 
 #include "layer.h"
 
@@ -201,10 +206,17 @@ ValueNode::subsys_init()
 	ADD_VALUENODE(ValueNode_Bone_Root,		  "bone_root",		  _("Root Bone"),		 RELEASE_VERSION_0_62_00); 
 	ADD_VALUENODE2(ValueNode_StaticList,	  "static_list",	  _("Static List"),		 RELEASE_VERSION_0_62_00); 
 	ADD_VALUENODE(ValueNode_BoneWeightPair,	  "boneweightpair",	  _("Bone Weight Pair"), RELEASE_VERSION_0_62_00); 
+	ADD_VALUENODE(ValueNode_BoneLink,		  "bone_link",		  _("Bone Link"),		 RELEASE_VERSION_1_0);
 
 	ADD_VALUENODE(ValueNode_WPList,           "wplist",           _("WPList"),           RELEASE_VERSION_0_63_00);
 	ADD_VALUENODE(ValueNode_DIList,           "dilist",           _("DIList"),           RELEASE_VERSION_0_63_01);
+
+	ADD_VALUENODE(ValueNode_Average,		  "average",		  _("Average"),			 RELEASE_VERSION_1_0);
+	ADD_VALUENODE(ValueNode_WeightedAverage,  "weighted_average", _("Weighted Average"), RELEASE_VERSION_1_0);
 	
+	ADD_VALUENODE(ValueNode_Dynamic,           "dynamic",         _("Dynamic"),          RELEASE_VERSION_1_0);
+	ADD_VALUENODE(ValueNode_Derivative,        "derivative",      _("Derivative"),       RELEASE_VERSION_1_0);
+
 #undef ADD_VALUENODE_CREATE
 #undef ADD_VALUENODE
 #undef ADD_VALUENODE2
@@ -219,7 +231,7 @@ ValueNode::subsys_stop()
 	return true;
 }
 
-ValueNode::ValueNode(ValueBase::Type type):type(type)
+ValueNode::ValueNode(Type &type):type(&type)
 {
 	value_node_count++;
 }
@@ -238,7 +250,7 @@ LinkableValueNode::create(const String &name, const ValueBase& x, Canvas::LooseH
 
 	if (!check_type(name, x.get_type()))
 	{
-		error(_("Bad type: ValueNode '%s' doesn't accept type '%s'"), book()[name].local_name.c_str(), ValueBase::type_local_name(x.get_type()).c_str());
+		error(_("Bad type: ValueNode '%s' doesn't accept type '%s'"), book()[name].local_name.c_str(), x.get_type().description.local_name.c_str());
 		return 0;
 	}
 
@@ -246,13 +258,13 @@ LinkableValueNode::create(const String &name, const ValueBase& x, Canvas::LooseH
 }
 
 bool
-LinkableValueNode::check_type(const String &name, ValueBase::Type x)
+LinkableValueNode::check_type(const String &name, Type &x)
 {
 	// the BoneRoot and Duplicate ValueNodes are exceptions - we don't want the
 	// user creating them for themselves, so check_type() fails for
 	// them even when it is valid
-	if((name == "bone_root" && x == ValueBase::TYPE_BONE) ||
-	   (name == "duplicate" && x == ValueBase::TYPE_REAL))
+	if((name == "bone_root" && x == type_bone_object) ||
+	   (name == "duplicate" && x == type_real))
 		return true;
 
 	if(!book().count(name) || !book()[name].check_type)
@@ -261,7 +273,7 @@ LinkableValueNode::check_type(const String &name, ValueBase::Type x)
 }
 
 bool
-LinkableValueNode::set_link(int i,ValueNode::Handle x, bool silent)
+LinkableValueNode::set_link(int i,ValueNode::Handle x)
 {
 	ValueNode::Handle previous(get_link(i));
 
@@ -290,7 +302,7 @@ LinkableValueNode::set_link(int i,ValueNode::Handle x, bool silent)
 		{
 			x->set_parent_canvas(get_parent_canvas());
 		}
-		if (!silent) changed();
+		changed();
 		return true;
 	}
 	return false;
@@ -546,7 +558,7 @@ PlaceholderValueNode::get_string()const
 	return String("PlaceholderValueNode: ") + get_guid().get_string();
 }
 
-ValueNode*
+ValueNode::Handle
 PlaceholderValueNode::clone(Canvas::LooseHandle canvas, const GUID& deriv_guid)const
 {
 	ValueNode* ret(new PlaceholderValueNode());
@@ -556,7 +568,7 @@ PlaceholderValueNode::clone(Canvas::LooseHandle canvas, const GUID& deriv_guid)c
 }
 
 PlaceholderValueNode::Handle
-PlaceholderValueNode::create(ValueBase::Type type)
+PlaceholderValueNode::create(Type &type)
 {
 	if (getenv("SYNFIG_DEBUG_PLACEHOLDER_VALUENODE"))
 		printf("%s:%d PlaceholderValueNode::create\n", __FILE__, __LINE__);
@@ -570,12 +582,12 @@ PlaceholderValueNode::operator()(Time /*t*/)const
 	return ValueBase();
 }
 
-PlaceholderValueNode::PlaceholderValueNode(ValueBase::Type type):
+PlaceholderValueNode::PlaceholderValueNode(Type &type):
 	ValueNode(type)
 {
 }
 
-ValueNode*
+ValueNode::Handle
 LinkableValueNode::clone(Canvas::LooseHandle canvas, const GUID& deriv_guid)const
 {
 	{
@@ -593,7 +605,7 @@ LinkableValueNode::clone(Canvas::LooseHandle canvas, const GUID& deriv_guid)cons
 		ValueNode::Handle link=get_link_vfunc(i);
 		if(!link->is_exported())
 		{
-			ValueNode::LooseHandle value_node(find_value_node(link->get_guid()^deriv_guid));
+			ValueNode::Handle value_node(find_value_node(link->get_guid()^deriv_guid));
 			if(!value_node)
 				value_node=link->clone(canvas, deriv_guid);
 			ret->set_link(i,value_node);
@@ -824,3 +836,11 @@ LinkableValueNode::set_children_vocab(const Vocab &newvocab)
 {
 	children_vocab.assign(newvocab.begin(),newvocab.end());
 }
+
+void
+LinkableValueNode::set_root_canvas(etl::loose_handle<Canvas> x)
+{
+	ValueNode::set_root_canvas(x);
+	for(int i = 0; i < link_count(); ++i)
+		get_link(i)->set_root_canvas(x);
+}
