@@ -51,19 +51,19 @@
 
 #define VALUENODE_CHECK_TYPE(type) 										\
 	/* I don't think this ever happens - maybe remove this code? */		\
-	if (get_type() == ValueBase::TYPE_NIL) {							\
+	if (get_type() == type_nil) {										\
 		warning("%s:%d get_type() IS nil sometimes!",					\
 				__FILE__, __LINE__);									\
 		return false;													\
 	}																	\
-	if (get_type() != ValueBase::TYPE_NIL &&							\
-		!(ValueBase::same_type_as(value->get_type(), type)) &&			\
+	if (get_type() != type_nil &&										\
+		!(ValueBase::can_copy(value->get_type(), type)) &&				\
 		!PlaceholderValueNode::Handle::cast_dynamic(value)) {			\
 		error(_("%s:%d wrong type for %s: need %s but got %s"),			\
 			  __FILE__, __LINE__,										\
 			  link_local_name(i).c_str(),								\
-			  ValueBase::type_local_name(type).c_str(),					\
-			  ValueBase::type_local_name(value->get_type()).c_str());	\
+			  type.description.local_name.c_str(),						\
+			  value->get_type().description.local_name.c_str() );		\
 		return false;													\
 	}
 
@@ -124,7 +124,7 @@ public:
 private:
 	//! The type of the Value Node
 	//! \see ValueBase
-	ValueBase::Type type;
+	Type *type;
 	//! The name of the Value Node. This is the string that is used in the
 	//! sif file to define the value type: i.e. <param name="amount">
 	String name;
@@ -187,7 +187,7 @@ public:
 
 protected:
 
-	ValueNode(ValueBase::Type type=ValueBase::TYPE_NIL);
+	ValueNode(Type &type=type_nil);
 
 public:
 
@@ -223,13 +223,13 @@ public:
 	String get_string()const;
 
 	//! Clones a Value Node
-	virtual ValueNode* clone(etl::loose_handle<Canvas> canvas, const GUID& deriv_guid=GUID())const=0;
+	virtual ValueNode::Handle clone(etl::loose_handle<Canvas> canvas, const GUID& deriv_guid=GUID())const=0;
 
 	//! Returns \true if the Value Node has an ID (has been exported)
 	bool is_exported()const { return !get_id().empty(); }
 
 	//! Returns the type of the ValueNode
-	ValueBase::Type get_type()const { return type; }
+	Type& get_type()const { return *type; }
 
 	//! Returns a handle to the parent canvas, if it has one.
 	etl::loose_handle<Canvas> get_parent_canvas()const;
@@ -262,7 +262,7 @@ public:
 
 protected:
 	//! Sets the type of the ValueNode
-	void set_type(ValueBase::Type t) { type=t; }
+	void set_type(Type &t) { type=&t; }
 
 	virtual void on_changed();
 }; // END of class ValueNode
@@ -285,7 +285,7 @@ public:
 
 private:
 
-	PlaceholderValueNode(ValueBase::Type type=ValueBase::TYPE_NIL);
+	PlaceholderValueNode(Type &type=type_nil);
 
 public:
 
@@ -297,9 +297,9 @@ public:
 
 	String get_string()const;
 
-	virtual ValueNode* clone(etl::loose_handle<Canvas> canvas, const GUID& deriv_guid=GUID())const;
+	virtual ValueNode::Handle clone(etl::loose_handle<Canvas> canvas, const GUID& deriv_guid=GUID())const;
 
-	static Handle create(ValueBase::Type type=ValueBase::TYPE_NIL);
+	static Handle create(Type &type=type_nil);
 
 protected:
 	virtual void get_times_vfunc(Node::time_set &/*set*/) const {}
@@ -338,7 +338,7 @@ public:
 	/*! As a pointer to the member, it represents a fucntion that checks
 	**  the type of the provided ValueBase
 	*/
-	typedef bool (*CheckType)(ValueBase::Type);
+	typedef bool (*CheckType)(Type &type);
 
 	struct BookEntry
 	{
@@ -371,10 +371,10 @@ public:
 	//! Each derived Linkable Value Node has to implement this fucntion and
 	//! should return true only if the type matches. \name is the name of
 	//! the linked value node and \x is the returned value type
-	static bool check_type(const String &name, ValueBase::Type x);
+	static bool check_type(const String &name, Type &x);
 
 public:
-	LinkableValueNode(ValueBase::Type type=ValueBase::TYPE_NIL):
+	LinkableValueNode(Type &type=type_nil):
 		ValueNode(type) { }
 
 protected:
@@ -402,10 +402,10 @@ public:
 	virtual int get_link_index_from_name(const String &name)const;
 
 	//! Clones a Value Node
-	virtual ValueNode* clone(etl::loose_handle<Canvas> canvas, const GUID& deriv_guid=GUID())const;
+	virtual ValueNode::Handle clone(etl::loose_handle<Canvas> canvas, const GUID& deriv_guid=GUID())const;
 
 	//! Sets a new Value Node link by its index
-	bool set_link(int i,ValueNode::Handle x,bool silent=false);
+	bool set_link(int i,ValueNode::Handle x);
 	//! Sets a new Value Node link by its name
 	bool set_link(const String &name,ValueNode::Handle x) {	return set_link(get_link_index_from_name(name),x);	}
 
@@ -422,11 +422,13 @@ public:
 	//! Gets the children vocabulary for linkable value nodes
 	virtual Vocab get_children_vocab()const;
 
+	virtual void set_root_canvas(etl::loose_handle<Canvas> x);
+
 protected:
 	//! Member to store the children vocabulary
 	Vocab children_vocab;
 	//! Sets the type of the ValueNode
-	void set_type(ValueBase::Type t) { ValueNode::set_type(t); }
+	void set_type(Type &t) { ValueNode::set_type(t); }
 
 	//! Virtual member to get the linked Value Node Handle
 	virtual ValueNode::LooseHandle get_link_vfunc(int i)const=0;
